package com.ebaiyihui.family.doctor.server.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.ebaiyihui.family.doctor.common.dto.SignedDoctorDTO;
import com.ebaiyihui.family.doctor.server.common.constants.CommonConstants;
import com.ebaiyihui.family.doctor.server.common.enums.StatusEnum;
import com.ebaiyihui.family.doctor.server.entity.PatientEntity;
import com.ebaiyihui.family.doctor.server.entity.PatientSignEntity;
import com.ebaiyihui.family.doctor.server.exception.BusinessException;
import com.ebaiyihui.family.doctor.server.mapper.PatientMapper;
import com.ebaiyihui.family.doctor.server.mapper.PatientSignMapper;
import com.ebaiyihui.family.doctor.server.service.ImChatTemplate;
import com.ebaiyihui.family.doctor.server.service.PatientService;
import com.ebaiyihui.family.doctor.server.util.IDCardUtil;
import com.ebaiyihui.family.doctor.server.util.UUIDUtil;
import com.ebaiyihui.framework.response.BaseResponse;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Objects;

/**
 * @ClassName: PatientServiceImpl
 * @Author:yanliang
 * @Date:2024/3/13 09:51
 * @Description
 */
@Service
@Slf4j
public class PatientServiceImpl implements PatientService {

    @Autowired
    private PatientSignMapper patientSignMapper;

    @Autowired
    private PatientMapper patientMapper;

    @Autowired
    private ImChatTemplate imChatTemplate;


    @Override
    public BaseResponse<String> signedDoctor(SignedDoctorDTO reqVo) {


        String orderId = UUIDUtil.generateViewId();
        String admId = UUIDUtil.getUUID();
        String patientId = "jtys_" + reqVo.getCredNo();
        String patientUserId = orderId + "_jtys";
        QueryWrapper<PatientEntity> wrapper = new QueryWrapper<>();
        PatientEntity patient = new PatientEntity();
        patient.setId(reqVo.getPatientId());
        wrapper.setEntity(patient);

        PatientEntity patientEntity = patientMapper.selectOne(wrapper);
        if (Objects.isNull(patientEntity)) {
            patientEntity = new PatientEntity();
            patientEntity.setId(patientId);
            patientEntity.setPatientName(reqVo.getName());
            patientEntity.setPhone(reqVo.getPhone());
            patientEntity.setPatientType(CommonConstants.PATIENT_TYPE);
            patientEntity.setGender(Integer.valueOf(IDCardUtil.getGenderForInteger(reqVo.getCredNo())));
            patientEntity.setBirthDates(IDCardUtil.getBirthByIdCard(reqVo.getCredNo()));
            patientEntity.setCredNo(reqVo.getCredNo());
            patientEntity.setUserId(patientUserId);
            log.info("数据库不存在添加患者: {}", JSON.toJSONString(patientEntity));
            patientMapper.insert(patientEntity);
        } else {
            patientId = patientEntity.getId();
            patientUserId = patientEntity.getUserId();
        }

        // 处理换绑医生
        if (Objects.nonNull(reqVo.getAdmId())) {
            QueryWrapper<PatientSignEntity> psWrapper = new QueryWrapper<>();
            PatientSignEntity patientSign = new PatientSignEntity();
            patientSign.setAdmId(admId);
            psWrapper.setEntity(patientSign);
            PatientSignEntity patientSignEntity = patientSignMapper.selectOne(psWrapper);
            if (Objects.nonNull(patientSignEntity)) {
                patientSignEntity.setStatus(StatusEnum.FINISH_APPLY.getValue());
                patientSignEntity.setSignStatus(3);
                patientSignEntity.setSubStatus(2);
                patientSignMapper.updateById(patientSignEntity);
            }
        }

        PatientSignEntity patientSignEntity = new PatientSignEntity();
        patientSignEntity.setAdmId(admId);
        patientSignEntity.setAppCode(CommonConstants.APP_CODE);
        patientSignEntity.setOrganId(CommonConstants.ORGAN_CODE);
        patientSignEntity.setOrganName(CommonConstants.ORGAN_NAME);
        patientSignEntity.setDoctorId(reqVo.getDoctorId());
        patientSignEntity.setDoctorName(reqVo.getDoctorName());
        patientSignEntity.setDeptId(reqVo.getDeptId());
        patientSignEntity.setDeptName(reqVo.getDeptName());
        patientSignEntity.setSignStatus(reqVo.getSignStatus());
        patientSignEntity.setPatientPhone(reqVo.getPhone());
        patientSignEntity.setCredNo(reqVo.getCredNo());
        patientSignEntity.setPatientId(patientId);
        patientSignEntity.setPatientName(reqVo.getName());
        patientSignEntity.setPatientUserId(patientUserId);
        patientSignEntity.setStatus(StatusEnum.IN_CONSULTATION.getValue());

        patientSignMapper.insert(patientSignEntity);

        //创建IM
        BaseResponse createImResponse = imChatTemplate.createImSession(patientSignEntity);
        if (!createImResponse.isSuccess()) {
            log.info("创建IM失败:{}", createImResponse.getMsg());
            throw new BusinessException("创建IM失败");
        }

//        if (2 == reqVo.getSignStatus()){
//            imChatTemplate.doctorPrompted(admId);
//        }


        return BaseResponse.success(admId);
    }

    @Override
    public BaseResponse<PatientSignEntity> getSignedDoctor(SignedDoctorDTO reqVo) {
        QueryWrapper<PatientSignEntity> psWrapper = new QueryWrapper<>();
        PatientSignEntity patientSign = new PatientSignEntity();
        patientSign.setAdmId(reqVo.getAdmId());
        psWrapper.setEntity(patientSign);
        PatientSignEntity patientSignEntity = patientSignMapper.selectOne(psWrapper);
        if (null == patientSignEntity) {
            throw new BusinessException("未查询到签约医生信息！");
        }
        return BaseResponse.success(patientSignEntity);
    }
}
